@@ -1,37 +1,36 @@
-# -*- coding: utf-8 -*-
-from hcloud.core.client import ClientEntityBase, BoundModelBase, GetEntityByNameMixin
-
 from hcloud.actions.client import BoundAction
+from hcloud.core.client import BoundModelBase, ClientEntityBase, GetEntityByNameMixin
 from hcloud.core.domain import add_meta_to_result
+from hcloud.datacenters.client import BoundDatacenter
 from hcloud.firewalls.client import BoundFirewall
 from hcloud.floating_ips.client import BoundFloatingIP
+from hcloud.images.client import BoundImage
+from hcloud.images.domain import CreateImageResponse
 from hcloud.isos.client import BoundIso
+from hcloud.networks.client import BoundNetwork  # noqa
+from hcloud.networks.domain import Network  # noqa
+from hcloud.placement_groups.client import BoundPlacementGroup
+from hcloud.primary_ips.client import BoundPrimaryIP
+from hcloud.server_types.client import BoundServerType
 from hcloud.servers.domain import (
-    Server,
     CreateServerResponse,
-    ResetPasswordResponse,
     EnableRescueResponse,
-    RequestConsoleResponse,
-    PublicNetwork,
     IPv4Address,
     IPv6Network,
     PrivateNet,
+    PublicNetwork,
     PublicNetworkFirewall,
+    RequestConsoleResponse,
+    ResetPasswordResponse,
+    Server,
 )
 from hcloud.volumes.client import BoundVolume
-from hcloud.images.domain import CreateImageResponse
-from hcloud.images.client import BoundImage
-from hcloud.server_types.client import BoundServerType
-from hcloud.datacenters.client import BoundDatacenter
-from hcloud.networks.client import BoundNetwork  # noqa
-from hcloud.networks.domain import Network  # noqa
 
 
 class BoundServer(BoundModelBase):
     model = Server
 
     def __init__(self, client, data, complete=True):
-
         datacenter = data.get("datacenter")
         if datacenter is not None:
             data["datacenter"] = BoundDatacenter(client._client.datacenters, datacenter)
@@ -60,8 +59,34 @@ class BoundServer(BoundModelBase):
 
         public_net = data.get("public_net")
         if public_net:
-            ipv4_address = IPv4Address.from_dict(public_net["ipv4"])
-            ipv6_network = IPv6Network.from_dict(public_net["ipv6"])
+            ipv4_address = (
+                IPv4Address.from_dict(public_net["ipv4"])
+                if public_net["ipv4"] is not None
+                else None
+            )
+            ipv4_primary_ip = (
+                BoundPrimaryIP(
+                    client._client.primary_ips,
+                    {"id": public_net["ipv4"]["id"]},
+                    complete=False,
+                )
+                if public_net["ipv4"] is not None
+                else None
+            )
+            ipv6_network = (
+                IPv6Network.from_dict(public_net["ipv6"])
+                if public_net["ipv6"] is not None
+                else None
+            )
+            ipv6_primary_ip = (
+                BoundPrimaryIP(
+                    client._client.primary_ips,
+                    {"id": public_net["ipv6"]["id"]},
+                    complete=False,
+                )
+                if public_net["ipv6"] is not None
+                else None
+            )
             floating_ips = [
                 BoundFloatingIP(
                     client._client.floating_ips, {"id": floating_ip}, complete=False
@@ -80,6 +105,8 @@ class BoundServer(BoundModelBase):
             data["public_net"] = PublicNetwork(
                 ipv4=ipv4_address,
                 ipv6=ipv6_network,
+                primary_ipv4=ipv4_primary_ip,
+                primary_ipv6=ipv6_primary_ip,
                 floating_ips=floating_ips,
                 firewalls=firewalls,
             )
@@ -101,7 +128,14 @@ class BoundServer(BoundModelBase):
             ]
             data["private_net"] = private_nets
 
-        super(BoundServer, self).__init__(client, data, complete)
+        placement_group = data.get("placement_group")
+        if placement_group:
+            placement_group = BoundPlacementGroup(
+                client._client.placement_groups, placement_group
+            )
+            data["placement_group"] = placement_group
+
+        super().__init__(client, data, complete)
 
     def get_actions_list(self, status=None, sort=None, page=None, per_page=None):
         # type: (Optional[List[str]], Optional[List[str]], Optional[int], Optional[int]) -> PageResults[List[BoundAction, Meta]]
@@ -355,6 +389,23 @@ class BoundServer(BoundModelBase):
         """
         return self._client.change_alias_ips(self, network, alias_ips)
 
+    def add_to_placement_group(self, placement_group):
+        # type: (Union[PlacementGroup,BoundPlacementGroup]) -> BoundAction
+        """Adds a server to a placement group.
+
+        :param placement_group: :class:`BoundPlacementGroup <hcloud.placement_groups.client.BoundPlacementGroup>` or :class:`Network <hcloud.placement_groups.domain.PlacementGroup>`
+        :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
+        """
+        return self._client.add_to_placement_group(self, placement_group)
+
+    def remove_from_placement_group(self):
+        # type: () -> BoundAction
+        """Removes a server from a placement group.
+
+        :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
+        """
+        return self._client.remove_from_placement_group(self)
+
 
 class ServersClient(ClientEntityBase, GetEntityByNameMixin):
     results_list_attribute_name = "servers"
@@ -366,9 +417,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :param id: int
         :return: :class:`BoundServer <hcloud.servers.client.BoundServer>`
         """
-        response = self._client.request(
-            url="/servers/{server_id}".format(server_id=id), method="GET"
-        )
+        response = self._client.request(url=f"/servers/{id}", method="GET")
         return BoundServer(self, response["server"])
 
     def get_list(
@@ -425,9 +474,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
                Can be used to filter servers by their status. The response will only contain servers matching the status.
         :return: List[:class:`BoundServer <hcloud.servers.client.BoundServer>`]
         """
-        return super(ServersClient, self).get_all(
-            name=name, label_selector=label_selector, status=status
-        )
+        return super().get_all(name=name, label_selector=label_selector, status=status)
 
     def get_by_name(self, name):
         # type: (str) -> BoundServer
@@ -437,7 +484,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
                Used to get server by name.
         :return: :class:`BoundServer <hcloud.servers.client.BoundServer>`
         """
-        return super(ServersClient, self).get_by_name(name)
+        return super().get_by_name(name)
 
     def create(
         self,
@@ -454,6 +501,8 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         datacenter=None,  # type: Optional[Datacenter]
         start_after_create=True,  # type: Optional[bool]
         automount=None,  # type: Optional[bool]
+        placement_group=None,  # type: Optional[PlacementGroup]
+        public_net=None,  # type: Optional[ServerCreatePublicNetwork]
     ):
         # type: (...) -> CreateServerResponse
         """Creates a new server. Returns preliminary information about the server as well as an action that covers progress of creation.
@@ -480,6 +529,10 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
                Start Server right after creation. Defaults to True.
         :param automount: boolean (optional)
                Auto mount volumes after attach.
+        :param placement_group: :class:`BoundPlacementGroup <hcloud.placement_groups.client.BoundPlacementGroup>` or :class:`Location <hcloud.placement_groups.domain.PlacementGroup>`
+               Placement Group where server should be added during creation
+        :param public_net: :class:`ServerCreatePublicNetwork <hcloud.servers.domain.ServerCreatePublicNetwork>`
+               Options to configure the public network of a server on creation
         :return: :class:`CreateServerResponse <hcloud.servers.domain.CreateServerResponse>`
         """
         data = {
@@ -507,6 +560,19 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
             data["labels"] = labels
         if automount is not None:
             data["automount"] = automount
+        if placement_group is not None:
+            data["placement_group"] = placement_group.id
+
+        if public_net is not None:
+            pn = {
+                "enable_ipv4": public_net.enable_ipv4,
+                "enable_ipv6": public_net.enable_ipv6,
+            }
+            if public_net.ipv4 is not None:
+                pn.update({"ipv4": public_net.ipv4.id})
+            if public_net.ipv6 is not None:
+                pn.update({"ipv6": public_net.ipv6.id})
+            data["public_net"] = pn
 
         response = self._client.request(url="/servers", method="POST", json=data)
 
@@ -549,7 +615,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
             params["per_page"] = per_page
 
         response = self._client.request(
-            url="/servers/{server_id}/actions".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions",
             method="GET",
             params=params,
         )
@@ -570,7 +636,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
                Specify how the results are sorted. Choices: `id` `id:asc` `id:desc` `command` `command:asc` `command:desc` `status` `status:asc` `status:desc` `progress` `progress:asc` `progress:desc` `started` `started:asc` `started:desc` `finished` `finished:asc` `finished:desc`
         :return: List[:class:`BoundAction <hcloud.actions.client.BoundAction>`]
         """
-        return super(ServersClient, self).get_actions(server, status=status, sort=sort)
+        return super().get_actions(server, status=status, sort=sort)
 
     def update(self, server, name=None, labels=None):
         # type:(Server,  Optional[str],  Optional[Dict[str, str]]) -> BoundServer
@@ -589,7 +655,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         if labels is not None:
             data.update({"labels": labels})
         response = self._client.request(
-            url="/servers/{server_id}".format(server_id=server.id),
+            url=f"/servers/{server.id}",
             method="PUT",
             json=data,
         )
@@ -602,9 +668,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :param server: :class:`BoundServer <hcloud.servers.client.BoundServer>` or :class:`Server <hcloud.servers.domain.Server>`
         :return:  :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        response = self._client.request(
-            url="/servers/{server_id}".format(server_id=server.id), method="DELETE"
-        )
+        response = self._client.request(url=f"/servers/{server.id}", method="DELETE")
         return BoundAction(self._client.actions, response["action"])
 
     def power_off(self, server):
@@ -615,7 +679,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :return:  :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/servers/{server_id}/actions/poweroff".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/poweroff",
             method="POST",
         )
         return BoundAction(self._client.actions, response["action"])
@@ -628,7 +692,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :return:  :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/servers/{server_id}/actions/poweron".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/poweron",
             method="POST",
         )
         return BoundAction(self._client.actions, response["action"])
@@ -641,7 +705,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :return:  :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/servers/{server_id}/actions/reboot".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/reboot",
             method="POST",
         )
         return BoundAction(self._client.actions, response["action"])
@@ -654,7 +718,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :return:  :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/servers/{server_id}/actions/reset".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/reset",
             method="POST",
         )
         return BoundAction(self._client.actions, response["action"])
@@ -667,7 +731,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :return:  :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/servers/{server_id}/actions/shutdown".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/shutdown",
             method="POST",
         )
         return BoundAction(self._client.actions, response["action"])
@@ -703,7 +767,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         """
         data = {"server_type": server_type.id_or_name, "upgrade_disk": upgrade_disk}
         response = self._client.request(
-            url="/servers/{server_id}/actions/change_type".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/change_type",
             method="POST",
             json=data,
         )
@@ -777,7 +841,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
             data.update({"labels": labels})
 
         response = self._client.request(
-            url="/servers/{server_id}/actions/create_image".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/create_image",
             method="POST",
             json=data,
         )
@@ -796,7 +860,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         """
         data = {"image": image.id_or_name}
         response = self._client.request(
-            url="/servers/{server_id}/actions/rebuild".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/rebuild",
             method="POST",
             json=data,
         )
@@ -842,7 +906,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         """
         data = {"iso": iso.id_or_name}
         response = self._client.request(
-            url="/servers/{server_id}/actions/attach_iso".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/attach_iso",
             method="POST",
             json=data,
         )
@@ -856,7 +920,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :return:  :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
         response = self._client.request(
-            url="/servers/{server_id}/actions/detach_iso".format(server_id=server.id),
+            url=f"/servers/{server.id}/actions/detach_iso",
             method="POST",
         )
         return BoundAction(self._client.actions, response["action"])
@@ -939,9 +1003,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
                 New alias IPs to set for this server.
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        data = {
-            "network": network.id,
-        }
+        data = {"network": network.id}
         if ip is not None:
             data.update({"ip": ip})
         if alias_ips is not None:
@@ -963,9 +1025,7 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
         :param network: :class:`BoundNetwork <hcloud.networks.client.BoundNetwork>` or :class:`Network <hcloud.networks.domain.Network>`
         :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
         """
-        data = {
-            "network": network.id,
-        }
+        data = {"network": network.id}
         response = self._client.request(
             url="/servers/{server_id}/actions/detach_from_network".format(
                 server_id=server.id
@@ -994,3 +1054,36 @@ class ServersClient(ClientEntityBase, GetEntityByNameMixin):
             json=data,
         )
         return BoundAction(self._client.actions, response["action"])
+
+    def add_to_placement_group(self, server, placement_group):
+        # type: (Union[Server,BoundServer], Union[PlacementGroup,BoundPlacementGroup]) -> BoundAction
+        """Adds a server to a placement group.
+
+        :param server: :class:`BoundServer <hcloud.servers.client.BoundServer>` or :class:`Server <hcloud.servers.domain.Server>`
+        :param placement_group: :class:`BoundPlacementGroup <hcloud.placement_groups.client.BoundPlacementGroup>` or :class:`Network <hcloud.placement_groups.domain.PlacementGroup>`
+        :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
+        """
+        data = {"placement_group": str(placement_group.id)}
+        response = self._client.request(
+            url="/servers/{server_id}/actions/add_to_placement_group".format(
+                server_id=server.id
+            ),
+            method="POST",
+            json=data,
+        )
+        return BoundAction(self._client.actions, response["action"])
+
+    def remove_from_placement_group(self, server):
+        # type: (Union[Server,BoundServer]) -> BoundAction
+        """Removes a server from a placement group.
+
+        :param server: :class:`BoundServer <hcloud.servers.client.BoundServer>` or :class:`Server <hcloud.servers.domain.Server>`
+        :return: :class:`BoundAction <hcloud.actions.client.BoundAction>`
+        """
+        response = self._client.request(
+            url="/servers/{server_id}/actions/remove_from_placement_group".format(
+                server_id=server.id
+            ),
+            method="POST",
+        )
+        return BoundAction(self._client.actions, response["action"])
